package thingml.xthingml.checks

import org.eclipse.xtext.validation.Check
import thingml.xthingml.constraints.ThingMLHelpers
import thingml.xthingml.constraints.Types
import thingml.xthingml.helpers.AnnotatedElementHelper
import thingml.xthingml.helpers.TyperHelper
import thingml.xthingml.thingML.Action
import thingml.xthingml.thingML.ConditionalAction
import thingml.xthingml.thingML.Expression
import thingml.xthingml.thingML.ExternExpression
import thingml.xthingml.thingML.LoopAction
import thingml.xthingml.xtext.validation.ThingMLValidatorCheck
import thingml.xthingml.xtext.validation.TypeChecker

class ControlStructures extends ThingMLValidatorCheck {
	
	def checkActionExpression(Action a, Expression e) {
		val actual = TypeChecker.computeTypeOf(e)
		if (actual.equals(Types.BOOLEAN_TYPE)) return;
		val t = ThingMLHelpers.findContainingThing(a);
		if (!AnnotatedElementHelper.isDefined(t, "ignore", "type-warning") && actual.equals(Types.ANY_TYPE)) {
			if (ThingMLHelpers.getAllExpressions(e, typeof(ExternExpression)).size > 0)
				warning("Condition involving extern expressions cannot be typed as Boolean. Consider using a cast, or use @ignore \"type-warning\" on the containing thing.", a, e.eContainingFeature, "type")
			else
				warning("Condition cannot be typed as Boolean. Consider using a cast, or use @ignore \"type-warning\" on the containing thing.", a, e.eContainingFeature, "type-cast", "Boolean")
		} else {
			error("Condition is not a Boolean ("+TyperHelper.getBroadType(actual).name+")", a, e.eContainingFeature, "type")
		}
	}
	
	@Check(FAST)
	def checkConditionalAction(ConditionalAction ca) {
		checkActionExpression(ca, ca.condition)
	}
	
	@Check(FAST)
	def checkLoopAction(LoopAction la) {
		checkActionExpression(la, la.condition)
	}
}