package thingml.xthingml.checks;

import org.eclipse.xtext.validation.Check;
import org.eclipse.xtext.validation.CheckType;
import thingml.xthingml.constraints.ThingMLHelpers;
import thingml.xthingml.constraints.Types;
import thingml.xthingml.helpers.AnnotatedElementHelper;
import thingml.xthingml.helpers.TyperHelper;
import thingml.xthingml.thingML.Action;
import thingml.xthingml.thingML.ConditionalAction;
import thingml.xthingml.thingML.Expression;
import thingml.xthingml.thingML.ExternExpression;
import thingml.xthingml.thingML.LoopAction;
import thingml.xthingml.thingML.Thing;
import thingml.xthingml.thingML.Type;
import thingml.xthingml.xtext.validation.ThingMLValidatorCheck;
import thingml.xthingml.xtext.validation.TypeChecker;

@SuppressWarnings("all")
public class ControlStructures extends ThingMLValidatorCheck {
  public void checkActionExpression(final Action a, final Expression e) {
    final Type actual = TypeChecker.computeTypeOf(e);
    boolean _equals = actual.equals(Types.BOOLEAN_TYPE);
    if (_equals) {
      return;
    }
    final Thing t = ThingMLHelpers.findContainingThing(a);
    if (((!AnnotatedElementHelper.isDefined(t, "ignore", "type-warning")) && actual.equals(Types.ANY_TYPE))) {
      int _size = ThingMLHelpers.<ExternExpression>getAllExpressions(e, ExternExpression.class).size();
      boolean _greaterThan = (_size > 0);
      if (_greaterThan) {
        this.warning("Condition involving extern expressions cannot be typed as Boolean. Consider using a cast, or use @ignore \"type-warning\" on the containing thing.", a, e.eContainingFeature(), "type");
      } else {
        this.warning("Condition cannot be typed as Boolean. Consider using a cast, or use @ignore \"type-warning\" on the containing thing.", a, e.eContainingFeature(), "type-cast", "Boolean");
      }
    } else {
      String _name = TyperHelper.getBroadType(actual).getName();
      String _plus = ("Condition is not a Boolean (" + _name);
      String _plus_1 = (_plus + ")");
      this.error(_plus_1, a, e.eContainingFeature(), "type");
    }
  }
  
  @Check(CheckType.FAST)
  public void checkConditionalAction(final ConditionalAction ca) {
    this.checkActionExpression(ca, ca.getCondition());
  }
  
  @Check(CheckType.FAST)
  public void checkLoopAction(final LoopAction la) {
    this.checkActionExpression(la, la.getCondition());
  }
}
