package thingml.xthingml.checks;

import com.google.common.base.Objects;
import java.util.List;
import java.util.function.Consumer;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.validation.Check;
import org.eclipse.xtext.validation.CheckType;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ListExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure2;
import thingml.xthingml.constraints.ThingMLHelpers;
import thingml.xthingml.constraints.Types;
import thingml.xthingml.helpers.ActionHelper;
import thingml.xthingml.helpers.AnnotatedElementHelper;
import thingml.xthingml.helpers.FunctionWithMultipleImplem;
import thingml.xthingml.helpers.ThingHelper;
import thingml.xthingml.helpers.TyperHelper;
import thingml.xthingml.helpers.UnimplementedFunction;
import thingml.xthingml.thingML.Action;
import thingml.xthingml.thingML.ActionBlock;
import thingml.xthingml.thingML.ConditionalAction;
import thingml.xthingml.thingML.Expression;
import thingml.xthingml.thingML.ExternStatement;
import thingml.xthingml.thingML.Function;
import thingml.xthingml.thingML.FunctionCallExpression;
import thingml.xthingml.thingML.FunctionCallStatement;
import thingml.xthingml.thingML.LoopAction;
import thingml.xthingml.thingML.Parameter;
import thingml.xthingml.thingML.PropertyReference;
import thingml.xthingml.thingML.ReturnAction;
import thingml.xthingml.thingML.Thing;
import thingml.xthingml.thingML.ThingMLModel;
import thingml.xthingml.thingML.ThingMLPackage;
import thingml.xthingml.thingML.Type;
import thingml.xthingml.thingML.Variable;
import thingml.xthingml.thingML.VariableAssignment;
import thingml.xthingml.xtext.validation.ThingMLValidatorCheck;
import thingml.xthingml.xtext.validation.TypeChecker;

@SuppressWarnings("all")
public class FunctionUsage extends ThingMLValidatorCheck {
  @Check(CheckType.FAST)
  public void checkThread(final Function f) {
    if ((AnnotatedElementHelper.isDefined(f, "fork_thread", "true") || AnnotatedElementHelper.hasFlag(f, "fork_thread"))) {
      if (((f.getTypeRef() != null) && (f.getTypeRef().getType() != null))) {
        String _name = f.getName();
        String _plus = ("Function " + _name);
        final String msg = (_plus + " forks a thread. It cannot return a result.");
        this.warning(msg, f, ThingMLPackage.eINSTANCE.getNamedElement_Name(), "function-thread-cannot-return", f.getName());
      }
    }
  }
  
  @Check(CheckType.NORMAL)
  public void checkUsage(final Function f) {
    final Thing thing = ThingMLHelpers.findContainingThing(f);
    final Function1<FunctionCallStatement, Boolean> _function = (FunctionCallStatement call) -> {
      Function _function_1 = call.getFunction();
      return Boolean.valueOf(Objects.equal(_function_1, f));
    };
    boolean _exists = IterableExtensions.<FunctionCallStatement>exists(ActionHelper.<FunctionCallStatement>getAllActions(thing, FunctionCallStatement.class), _function);
    if (_exists) {
      return;
    }
    final Function1<FunctionCallExpression, Boolean> _function_1 = (FunctionCallExpression call) -> {
      Function _function_2 = call.getFunction();
      return Boolean.valueOf(Objects.equal(_function_2, f));
    };
    boolean _exists_1 = IterableExtensions.<FunctionCallExpression>exists(ThingMLHelpers.<FunctionCallExpression>getAllExpressions(thing, FunctionCallExpression.class), _function_1);
    if (_exists_1) {
      return;
    }
    final ThingMLModel model = ThingMLHelpers.findContainingModel(thing);
    final Function1<Thing, Boolean> _function_2 = (Thing t) -> {
      final Function1<Thing, Boolean> _function_3 = (Thing t2) -> {
        return Boolean.valueOf(Objects.equal(t2, thing));
      };
      return Boolean.valueOf(IterableExtensions.<Thing>exists(ThingHelper.allIncludedThings(t), _function_3));
    };
    final Function1<Thing, Boolean> _function_3 = (Thing t) -> {
      return Boolean.valueOf((IterableExtensions.<FunctionCallStatement>exists(ActionHelper.<FunctionCallStatement>getAllActions(t, FunctionCallStatement.class), ((Function1<FunctionCallStatement, Boolean>) (FunctionCallStatement call) -> {
        String _name = call.getFunction().getName();
        String _name_1 = f.getName();
        return Boolean.valueOf(Objects.equal(_name, _name_1));
      })) || IterableExtensions.<FunctionCallExpression>exists(ThingMLHelpers.<FunctionCallExpression>getAllExpressions(t, FunctionCallExpression.class), ((Function1<FunctionCallExpression, Boolean>) (FunctionCallExpression call) -> {
        String _name = call.getFunction().getName();
        String _name_1 = f.getName();
        return Boolean.valueOf(Objects.equal(_name, _name_1));
      }))));
    };
    final boolean isUsedByIncludingThings = IterableExtensions.<Thing>exists(IterableExtensions.<Thing>filter(ThingMLHelpers.allThings(model), _function_2), _function_3);
    final boolean isUsed = (isUsedByIncludingThings || IterableExtensions.<Thing>exists(ThingHelper.allIncludedThings(thing), ((Function1<Thing, Boolean>) (Thing t) -> {
      return Boolean.valueOf((IterableExtensions.<FunctionCallStatement>exists(ActionHelper.<FunctionCallStatement>getAllActions(t, FunctionCallStatement.class), ((Function1<FunctionCallStatement, Boolean>) (FunctionCallStatement call) -> {
        String _name = call.getFunction().getName();
        String _name_1 = f.getName();
        return Boolean.valueOf(Objects.equal(_name, _name_1));
      })) || IterableExtensions.<FunctionCallExpression>exists(ThingMLHelpers.<FunctionCallExpression>getAllExpressions(t, FunctionCallExpression.class), ((Function1<FunctionCallExpression, Boolean>) (FunctionCallExpression call) -> {
        String _name = call.getFunction().getName();
        String _name_1 = f.getName();
        return Boolean.valueOf(Objects.equal(_name, _name_1));
      }))));
    })));
    if ((!isUsed)) {
      String _name = f.getName();
      String _plus = ("Function " + _name);
      final String msg = (_plus + " is never called.");
      this.warning(msg, f, ThingMLPackage.eINSTANCE.getNamedElement_Name(), "function-never-called", f.getName());
    }
  }
  
  @Check(CheckType.FAST)
  public void checkConcrete(final Thing t) {
    boolean _isFragment = t.isFragment();
    if (_isFragment) {
      return;
    }
    final Function1<Function, Boolean> _function = (Function f) -> {
      return Boolean.valueOf(f.isAbstract());
    };
    final Consumer<Function> _function_1 = (Function f) -> {
      try {
        try {
          ThingHelper.getConcreteFunction(t, f);
        } catch (final Throwable _t) {
          if (_t instanceof UnimplementedFunction) {
            final UnimplementedFunction e = (UnimplementedFunction)_t;
            final Thing origin = ThingMLHelpers.findContainingThing(f);
            boolean _equals = Objects.equal(origin, t);
            if (_equals) {
              this.error(e.getMessage(), f, ThingMLPackage.eINSTANCE.getNamedElement_Name(), "function-not-implemented", f.getName());
            } else {
              this.error(e.getMessage(), t, ThingMLPackage.eINSTANCE.getThing_Includes(), t.getIncludes().indexOf(origin), "function-not-implemented", f.getName());
            }
          } else if (_t instanceof FunctionWithMultipleImplem) {
            final FunctionWithMultipleImplem e_1 = (FunctionWithMultipleImplem)_t;
            final Thing origin_1 = ThingMLHelpers.findContainingThing(f);
            boolean _equals_1 = Objects.equal(origin_1, t);
            if (_equals_1) {
              this.error(e_1.getMessage(), t, ThingMLPackage.eINSTANCE.getNamedElement_Name(), "function-many-impl", f.getName());
            } else {
              this.error(e_1.getMessage(), t, ThingMLPackage.eINSTANCE.getThing_Includes(), t.getIncludes().indexOf(origin_1), "function-many-impl", f.getName());
            }
          } else {
            throw Exceptions.sneakyThrow(_t);
          }
        }
      } catch (Throwable _e) {
        throw Exceptions.sneakyThrow(_e);
      }
    };
    IterableExtensions.<Function>filter(ThingMLHelpers.allFunctions(t), _function).forEach(_function_1);
  }
  
  @Check(CheckType.FAST)
  public void checkAbstract(final Function f) {
    EObject _eContainer = f.eContainer();
    Thing thing = ((Thing) _eContainer);
    if (((!thing.isFragment()) && f.isAbstract())) {
      String _name = thing.getName();
      String _plus = ("Thing " + _name);
      String _plus_1 = (_plus + " is not a fragment, so it cannot contain abstract functions");
      this.error(_plus_1, thing, f.eContainingFeature(), thing.getFunctions().indexOf(f), "abstract-function-fragment", f.getName());
    }
  }
  
  @Check(CheckType.NORMAL)
  public void checkParameters(final Function f) {
    boolean _isAbstract = f.isAbstract();
    boolean _not = (!_isAbstract);
    if (_not) {
      final List<PropertyReference> refs = ThingMLHelpers.<PropertyReference>getAllExpressions(f, PropertyReference.class);
      final List<VariableAssignment> assigns = ActionHelper.<VariableAssignment>getAllActions(f, VariableAssignment.class);
      final Procedure2<Parameter, Integer> _function = (Parameter p, Integer i) -> {
        final Function1<PropertyReference, Boolean> _function_1 = (PropertyReference pr) -> {
          Variable _property = pr.getProperty();
          return Boolean.valueOf((_property == p));
        };
        final boolean isUsed = IterableExtensions.<PropertyReference>exists(refs, _function_1);
        if ((!isUsed)) {
          String _name = p.getName();
          String _plus = ("Parameter " + _name);
          String _plus_1 = (_plus + " is never used");
          this.warning(_plus_1, f, ThingMLPackage.eINSTANCE.getFunction_Parameters(), (i).intValue(), "parameter-not-used");
        }
        final Consumer<VariableAssignment> _function_2 = (VariableAssignment va) -> {
          Variable _property = va.getProperty();
          boolean _tripleEquals = (_property == p);
          if (_tripleEquals) {
            String _name_1 = p.getName();
            String _plus_2 = ("Re-assigning parameter " + _name_1);
            String _plus_3 = (_plus_2 + " can have side effects");
            this.warning(_plus_3, va.eContainer(), va.eContainingFeature(), "parameter-reassign");
          }
        };
        assigns.forEach(_function_2);
      };
      IterableExtensions.<Parameter>forEach(f.getParameters(), _function);
    }
  }
  
  private boolean returns(final Action a) {
    if ((a instanceof ReturnAction)) {
      return true;
    }
    if ((a instanceof ActionBlock)) {
      final ActionBlock block = ((ActionBlock) a);
      final Function1<Action, Boolean> _function = (Action aa) -> {
        return Boolean.valueOf(this.returns(aa));
      };
      return IterableExtensions.<Action>exists(ListExtensions.<Action>reverseView(block.getActions()), _function);
    }
    if ((a instanceof ConditionalAction)) {
      final ConditionalAction ca = ((ConditionalAction) a);
      return ((this.returns(ca.getAction()) && (ca.getElseAction() != null)) && this.returns(ca.getElseAction()));
    }
    if ((a instanceof LoopAction)) {
      final LoopAction la = ((LoopAction) a);
      return this.returns(la.getAction());
    }
    return false;
  }
  
  @Check(CheckType.FAST)
  public void checkReturnOnlyInFunction(final ReturnAction r) {
    EObject _eContainer = r.eContainer();
    EObject parent = ((EObject) _eContainer);
    while (((parent != null) && (!(parent instanceof Function)))) {
      EObject _eContainer_1 = parent.eContainer();
      parent = ((EObject) _eContainer_1);
    }
    if (((parent == null) || (((Function) parent).getTypeRef() == null))) {
      final Object c = r.eContainer().eGet(r.eContainingFeature());
      final String msg = "Return action is only allowed in functions with a return type.";
      if ((c instanceof EList)) {
        this.error(msg, r.eContainer(), r.eContainingFeature(), ((EList) c).indexOf(r), "return-not-allowed");
      } else {
        this.error(msg, r.eContainer(), r.eContainingFeature(), "return-not-allowed");
      }
    }
  }
  
  private int indexOfLastExtern(final ActionBlock block) {
    final Function1<Action, Boolean> _function = (Action a) -> {
      return Boolean.valueOf((a instanceof ExternStatement));
    };
    Action last = IterableExtensions.<Action>findLast(block.getActions(), _function);
    if ((last != null)) {
      return block.getActions().indexOf(last);
    }
    return (-1);
  }
  
  @Check(CheckType.FAST)
  public void checkBlock(final ActionBlock block) {
    Function _xifexpression = null;
    EObject _eContainer = block.eContainer();
    if ((_eContainer instanceof Function)) {
      EObject _eContainer_1 = block.eContainer();
      _xifexpression = ((Function) _eContainer_1);
    } else {
      _xifexpression = null;
    }
    final Function f = _xifexpression;
    if ((((f == null) || (f.getTypeRef() == null)) || (f.getTypeRef().getType() == null))) {
      return;
    }
    final int indexOfLastExtern = this.indexOfLastExtern(block);
    int indexOfFirstReturn = (-1);
    if ((indexOfLastExtern < 0)) {
      final Function1<Action, Boolean> _function = (Action a) -> {
        return Boolean.valueOf(this.returns(a));
      };
      final Action firstReturn = IterableExtensions.<Action>findFirst(block.getActions(), _function);
      indexOfFirstReturn = block.getActions().indexOf(firstReturn);
    } else {
      return;
    }
    int _size = block.getActions().size();
    int _minus = (_size - 1);
    boolean _lessThan = (indexOfFirstReturn < _minus);
    if (_lessThan) {
      final String msg = "The code from here and below is unreachable (the code above will return in any case)";
      this.error(msg, block, ThingMLPackage.eINSTANCE.getActionBlock_Actions(), (indexOfFirstReturn + 1), "unreachable-code", Integer.valueOf(indexOfFirstReturn).toString());
    }
  }
  
  @Check(CheckType.FAST)
  public void checkReturnType2(final Function f) {
    if ((((!f.isAbstract()) && (f.getTypeRef() != null)) && (f.getTypeRef().getType() != null))) {
      if (((f.getBody() instanceof ReturnAction) || ((f.getBody() instanceof ConditionalAction) && this.returns(f.getBody())))) {
        return;
      }
      boolean _returns = this.returns(f.getBody());
      boolean _not = (!_returns);
      if (_not) {
        final Type actualType = TyperHelper.getBroadType(f.getTypeRef().getType());
        String _name = f.getName();
        String _plus = ("Function " + _name);
        String _plus_1 = (_plus + " must return ");
        String _name_1 = actualType.getName();
        String _plus_2 = (_plus_1 + _name_1);
        final String msg = (_plus_2 + ". Found no return action.");
        EObject _eContainer = f.eContainer();
        this.error(msg, f.eContainer(), f.eContainingFeature(), ((Thing) _eContainer).getFunctions().indexOf(f), "missing-return");
        return;
      }
    }
  }
  
  @Check(CheckType.FAST)
  public void checkReturnType(final Function f) {
    boolean _isAbstract = f.isAbstract();
    boolean _not = (!_isAbstract);
    if (_not) {
      if (((f.getTypeRef() != null) && (f.getTypeRef().getType() != null))) {
        final Consumer<ReturnAction> _function = (ReturnAction ra) -> {
          final Type actualType = TyperHelper.getBroadType(f.getTypeRef().getType());
          final Type returnType = TypeChecker.computeTypeOf(ra.getExp());
          final Object parent = ra.eContainer().eGet(ra.eContainingFeature());
          boolean _equals = returnType.equals(Types.ERROR_TYPE);
          if (_equals) {
            String _name = f.getName();
            String _plus = ("Function " + _name);
            String _plus_1 = (_plus + " should return ");
            String _name_1 = actualType.getName();
            String _plus_2 = (_plus_1 + _name_1);
            String _plus_3 = (_plus_2 + ". Found ");
            String _name_2 = returnType.getName();
            String _plus_4 = (_plus_3 + _name_2);
            final String msg = (_plus_4 + ".");
            if ((parent instanceof EList)) {
              this.error(msg, ra.eContainer(), ra.eContainingFeature(), ((EList) parent).indexOf(ra), "type");
            } else {
              this.error(msg, ra.eContainer(), ra.eContainingFeature(), "type");
            }
          } else {
            boolean _equals_1 = returnType.equals(Types.ANY_TYPE);
            if (_equals_1) {
              String _name_3 = f.getName();
              String _plus_5 = ("Function " + _name_3);
              String _plus_6 = (_plus_5 + " should return ");
              String _name_4 = actualType.getName();
              String _plus_7 = (_plus_6 + _name_4);
              final String msg_1 = (_plus_7 + ". Found a value/expression that cannot be typed. Consider using a cast (<exp> as <type>).");
              if ((parent instanceof EList)) {
                this.warning(msg_1, ra.eContainer(), ra.eContainingFeature(), ((EList) parent).indexOf(ra), "type-cast", f.getTypeRef().getType().getName());
              } else {
                this.warning(msg_1, ra.eContainer(), ra.eContainingFeature(), "type-cast", f.getTypeRef().getType().getName());
              }
            } else {
              boolean _isA = TyperHelper.isA(returnType, actualType);
              boolean _not_1 = (!_isA);
              if (_not_1) {
                String _name_5 = f.getName();
                String _plus_8 = ("Function " + _name_5);
                String _plus_9 = (_plus_8 + " should return ");
                String _name_6 = actualType.getName();
                String _plus_10 = (_plus_9 + _name_6);
                String _plus_11 = (_plus_10 + ". Found ");
                String _name_7 = returnType.getName();
                String _plus_12 = (_plus_11 + _name_7);
                final String msg_2 = (_plus_12 + ".");
                if ((parent instanceof EList)) {
                  this.error(msg_2, ra.eContainer(), ra.eContainingFeature(), ((EList) parent).indexOf(ra), "type");
                } else {
                  this.error(msg_2, ra.eContainer(), ra.eContainingFeature(), "type");
                }
              }
            }
          }
        };
        ActionHelper.<ReturnAction>getAllActions(f, ReturnAction.class).forEach(_function);
      }
    }
  }
  
  @Check(CheckType.FAST)
  public void checkFunctionCallAction(final FunctionCallStatement call) {
    final Thing t = ThingMLHelpers.findContainingThing(call);
    final boolean ignore = (AnnotatedElementHelper.isDefined(t, "ignore", "type-warning") || AnnotatedElementHelper.isDefined(call.getFunction(), "ignore", "type-warning"));
    this.checkFunctionCall(call.getFunction(), call.getParameters(), call, ignore);
  }
  
  @Check(CheckType.FAST)
  public void checkFunctionCallExpression(final FunctionCallExpression call) {
    final Thing t = ThingMLHelpers.findContainingThing(call);
    final boolean ignore = (AnnotatedElementHelper.isDefined(t, "ignore", "type-warning") || AnnotatedElementHelper.isDefined(call.getFunction(), "ignore", "type-warning"));
    this.checkFunctionCall(call.getFunction(), call.getParameters(), call, ignore);
  }
  
  public void checkFunctionCall(final Function function, final List<Expression> params, final EObject o, final boolean ignore) {
    final Object parent = o.eContainer().eGet(o.eContainingFeature());
    int _size = function.getParameters().size();
    int _size_1 = params.size();
    boolean _tripleNotEquals = (_size != _size_1);
    if (_tripleNotEquals) {
      String _name = function.getName();
      String _plus = ("Function " + _name);
      String _plus_1 = (_plus + " is called with a wrong number of arguments. Expected ");
      int _size_2 = function.getParameters().size();
      String _plus_2 = (_plus_1 + Integer.valueOf(_size_2));
      String _plus_3 = (_plus_2 + ", called with ");
      int _size_3 = params.size();
      final String msg = (_plus_3 + Integer.valueOf(_size_3));
      if ((parent instanceof EList)) {
        this.error(msg, o.eContainer(), o.eContainingFeature(), ((EList) parent).indexOf(o), "function-call-wrong-parameter-length");
      } else {
        this.error(msg, o.eContainer(), o.eContainingFeature(), "function-call-wrong-parameter-length");
      }
      return;
    }
    final Procedure2<Parameter, Integer> _function = (Parameter p, Integer i) -> {
      final Expression e = params.get((i).intValue());
      final Type expected = TyperHelper.getBroadType(p.getTypeRef().getType());
      final Type actual = TypeChecker.computeTypeOf(e);
      if ((actual != null)) {
        boolean _equals = actual.equals(Types.ERROR_TYPE);
        if (_equals) {
          String _name_1 = function.getName();
          String _plus_4 = ("Function " + _name_1);
          String _plus_5 = (_plus_4 + " is called with an erroneous parameter. Expected ");
          String _name_2 = expected.getName();
          String _plus_6 = (_plus_5 + _name_2);
          String _plus_7 = (_plus_6 + ", called with ");
          String _name_3 = TyperHelper.getBroadType(actual).getName();
          final String msg_1 = (_plus_7 + _name_3);
          if ((parent instanceof EList)) {
            this.error(msg_1, o.eContainer(), o.eContainingFeature(), ((EList) parent).indexOf(o), "function-call-wrong-parameter-type");
          } else {
            this.error(msg_1, o.eContainer(), o.eContainingFeature(), "function-call-wrong-parameter-type");
          }
        } else {
          if (((!ignore) && actual.equals(Types.ANY_TYPE))) {
            String _name_4 = function.getName();
            String _plus_8 = ("Function " + _name_4);
            final String msg_2 = (_plus_8 + " is called with a parameter which cannot be typed. Consider using a cast (<exp> as <type>), or using @ignore \"type-warning\".");
            if ((parent instanceof EList)) {
              this.warning(msg_2, o.eContainer(), o.eContainingFeature(), ((EList) parent).indexOf(o), "function-call-wrong-parameter-type");
            } else {
              this.warning(msg_2, o.eContainer(), o.eContainingFeature(), "function-call-wrong-parameter-type");
            }
          } else {
            boolean _isA = TyperHelper.isA(actual, expected);
            boolean _not = (!_isA);
            if (_not) {
              String _name_5 = function.getName();
              String _plus_9 = ("Function " + _name_5);
              String _plus_10 = (_plus_9 + " is called with an erroneous parameter. Expected ");
              String _name_6 = expected.getName();
              String _plus_11 = (_plus_10 + _name_6);
              String _plus_12 = (_plus_11 + ", called with ");
              String _name_7 = TyperHelper.getBroadType(actual).getName();
              final String msg_3 = (_plus_12 + _name_7);
              if ((parent instanceof EList)) {
                this.error(msg_3, o.eContainer(), o.eContainingFeature(), ((EList) parent).indexOf(o), "function-call-wrong-parameter-type");
              } else {
                this.error(msg_3, o.eContainer(), o.eContainingFeature(), "function-call-wrong-parameter-type");
              }
            }
          }
        }
      }
    };
    IterableExtensions.<Parameter>forEach(function.getParameters(), _function);
  }
}
