package thingml.xthingml.checks;

import java.util.HashSet;
import java.util.Set;
import java.util.function.Consumer;
import java.util.function.Predicate;
import org.eclipse.xtext.validation.Check;
import org.eclipse.xtext.validation.CheckType;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure2;
import thingml.xthingml.constraints.ThingMLHelpers;
import thingml.xthingml.helpers.TyperHelper;
import thingml.xthingml.thingML.ConfigPropertyAssign;
import thingml.xthingml.thingML.Configuration;
import thingml.xthingml.thingML.Enumeration;
import thingml.xthingml.thingML.EnumerationLiteral;
import thingml.xthingml.thingML.Expression;
import thingml.xthingml.thingML.Instance;
import thingml.xthingml.thingML.Property;
import thingml.xthingml.thingML.PropertyAssign;
import thingml.xthingml.thingML.Thing;
import thingml.xthingml.thingML.ThingMLModel;
import thingml.xthingml.thingML.ThingMLPackage;
import thingml.xthingml.thingML.Type;
import thingml.xthingml.thingML.TypeRef;
import thingml.xthingml.xtext.validation.ThingMLValidatorCheck;
import thingml.xthingml.xtext.validation.TypeChecker;

@SuppressWarnings("all")
public class PropertyInitialization extends ThingMLValidatorCheck {
  public Set<Property> getUninitializedProperties(final Thing t) {
    final HashSet<Property> props = CollectionLiterals.<Property>newHashSet();
    final Consumer<Property> _function = (Property prop) -> {
      Expression _init = prop.getInit();
      boolean _tripleEquals = (_init == null);
      if (_tripleEquals) {
        props.add(prop);
      }
    };
    t.getProperties().forEach(_function);
    final Consumer<Thing> _function_1 = (Thing inc) -> {
      props.addAll(this.getUninitializedProperties(inc));
    };
    t.getIncludes().forEach(_function_1);
    final Consumer<PropertyAssign> _function_2 = (PropertyAssign propAssign) -> {
      final Predicate<Property> _function_3 = (Property prop) -> {
        Property _property = propAssign.getProperty();
        return (prop == _property);
      };
      props.removeIf(_function_3);
    };
    t.getAssign().forEach(_function_2);
    return props;
  }
  
  @Check(CheckType.FAST)
  public void checkPropertyInitialization(final Configuration cfg) {
    final Procedure2<Instance, Integer> _function = (Instance inst, Integer i) -> {
      final Set<Property> props = this.getUninitializedProperties(inst.getType());
      final Consumer<ConfigPropertyAssign> _function_1 = (ConfigPropertyAssign propAssign) -> {
        final Predicate<Property> _function_2 = (Property prop) -> {
          Property _property = propAssign.getProperty();
          return (prop == _property);
        };
        props.removeIf(_function_2);
      };
      cfg.getPropassigns().forEach(_function_1);
      boolean _isEmpty = props.isEmpty();
      boolean _not = (!_isEmpty);
      if (_not) {
        final Function1<Property, CharSequence> _function_2 = (Property it) -> {
          return it.getName();
        };
        final String msg = IterableExtensions.<Property>join(props, "Properties (", ", ", ") are not initialized", _function_2);
        this.warning(msg, cfg, ThingMLPackage.eINSTANCE.getConfiguration_Instances(), (i).intValue(), "properties-not-initialized");
      }
    };
    IterableExtensions.<Instance>forEach(cfg.getInstances(), _function);
  }
  
  @Check(CheckType.FAST)
  public void checkEnumInitialization(final Enumeration e) {
    TypeRef _typeRef = e.getTypeRef();
    boolean _tripleNotEquals = (_typeRef != null);
    if (_tripleNotEquals) {
      final Function1<EnumerationLiteral, Boolean> _function = (EnumerationLiteral l) -> {
        Expression _init = l.getInit();
        return Boolean.valueOf((_init == null));
      };
      boolean _exists = IterableExtensions.<EnumerationLiteral>exists(e.getLiterals(), _function);
      if (_exists) {
        String _name = e.getName();
        String _plus = ("Enumeration " + _name);
        final String msg = (_plus + " is typed. All literals must be initialized.");
        ThingMLModel _findContainingModel = ThingMLHelpers.findContainingModel(e);
        this.error(msg, ThingMLHelpers.findContainingModel(e), ThingMLPackage.eINSTANCE.getThingMLModel_Types(), ((ThingMLModel) _findContainingModel).getTypes().indexOf(e));
      }
      final Procedure2<EnumerationLiteral, Integer> _function_1 = (EnumerationLiteral l, Integer i) -> {
        Expression _init = l.getInit();
        boolean _tripleNotEquals_1 = (_init != null);
        if (_tripleNotEquals_1) {
          final Type litType = TypeChecker.computeTypeOf(l.getInit());
          boolean _isA = TyperHelper.isA(litType, e.getTypeRef().getType());
          boolean _not = (!_isA);
          if (_not) {
            String _name_1 = l.getName();
            String _plus_1 = ("Literal " + _name_1);
            String _plus_2 = (_plus_1 + " must be of type ");
            String _name_2 = TyperHelper.getBroadType(e.getTypeRef().getType()).getName();
            String _plus_3 = (_plus_2 + _name_2);
            String _plus_4 = (_plus_3 + ". Found ");
            String _name_3 = TyperHelper.getBroadType(litType).getName();
            final String msg_1 = (_plus_4 + _name_3);
            this.error(msg_1, e, ThingMLPackage.eINSTANCE.getEnumeration_Literals(), (i).intValue());
          }
        }
      };
      IterableExtensions.<EnumerationLiteral>forEach(e.getLiterals(), _function_1);
    }
  }
}
