package thingml.xthingml.checks;

import java.util.List;
import java.util.function.Consumer;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.xtext.validation.Check;
import org.eclipse.xtext.validation.CheckType;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure2;
import thingml.xthingml.constraints.ThingMLHelpers;
import thingml.xthingml.constraints.Types;
import thingml.xthingml.helpers.AnnotatedElementHelper;
import thingml.xthingml.helpers.CompositeStateHelper;
import thingml.xthingml.helpers.ThingHelper;
import thingml.xthingml.helpers.TyperHelper;
import thingml.xthingml.thingML.Action;
import thingml.xthingml.thingML.CastExpression;
import thingml.xthingml.thingML.CompositeState;
import thingml.xthingml.thingML.Expression;
import thingml.xthingml.thingML.LocalVariable;
import thingml.xthingml.thingML.Property;
import thingml.xthingml.thingML.Thing;
import thingml.xthingml.thingML.ThingMLPackage;
import thingml.xthingml.thingML.Type;
import thingml.xthingml.thingML.Variable;
import thingml.xthingml.thingML.VariableAssignment;
import thingml.xthingml.xtext.validation.ThingMLValidatorCheck;
import thingml.xthingml.xtext.validation.TypeChecker;

@SuppressWarnings("all")
public class VariableUsage extends ThingMLValidatorCheck {
  @Check(CheckType.FAST)
  public void checkCast(final CastExpression cast) {
    final Type actual = TypeChecker.computeTypeOf(cast.getTerm());
    boolean _isA = TyperHelper.isA(actual, cast.getType());
    boolean _not = (!_isA);
    if (_not) {
      String _name = actual.getName();
      String _plus = ("Cannot cast " + _name);
      String _plus_1 = (_plus + " to ");
      String _name_1 = cast.getType().getName();
      final String msg = (_plus_1 + _name_1);
      final Object parent = cast.eContainer().eGet(cast.eContainingFeature());
      if ((parent instanceof EList)) {
        this.error(msg, cast.eContainer(), cast.eContainingFeature(), ((EList<Action>) parent).indexOf(cast), "type");
      } else {
        this.error(msg, cast.eContainer(), cast.eContainingFeature(), "type");
      }
    }
  }
  
  public void checkType(final Variable va, final Expression e, final EObject o, final EStructuralFeature f) {
    Expression _cardinality = va.getTypeRef().getCardinality();
    boolean _tripleEquals = (_cardinality == null);
    if (_tripleEquals) {
      final Type expected = TyperHelper.getBroadType(va.getTypeRef().getType());
      final Type actual = TypeChecker.computeTypeOf(e);
      if ((actual != null)) {
        final Type broad = TyperHelper.getBroadType(actual);
        final Thing t = ThingMLHelpers.findContainingThing(va);
        final boolean ignore = (AnnotatedElementHelper.isDefined(t, "ignore", "type-warning") || AnnotatedElementHelper.isDefined(va, "ignore", "type-warning"));
        boolean _equals = actual.equals(Types.ERROR_TYPE);
        if (_equals) {
          String _name = va.getName();
          String _plus = ("Property " + _name);
          String _plus_1 = (_plus + " is assigned with an erroneous value/expression. Expected ");
          String _name_1 = expected.getName();
          String _plus_2 = (_plus_1 + _name_1);
          String _plus_3 = (_plus_2 + ", assigned with ");
          String _name_2 = broad.getName();
          final String msg = (_plus_3 + _name_2);
          this.error(msg, o, f, "type");
        } else {
          if (((!ignore) && actual.equals(Types.ANY_TYPE))) {
            String _name_3 = va.getName();
            String _plus_4 = ("Property " + _name_3);
            final String msg_1 = (_plus_4 + " is assigned with a value/expression which cannot be typed. Consider using a cast (<exp> as <Type>), or use @ignore \"type-warning\"");
            this.warning(msg_1, o, f, "type-cast", va.getTypeRef().getType().getName());
          } else {
            boolean _isA = TyperHelper.isA(actual, expected);
            boolean _not = (!_isA);
            if (_not) {
              String _name_4 = va.getName();
              String _plus_5 = ("Property " + _name_4);
              String _plus_6 = (_plus_5 + " is assigned with an erroneous value/expression. Expected ");
              String _name_5 = expected.getName();
              String _plus_7 = (_plus_6 + _name_5);
              String _plus_8 = (_plus_7 + ", assigned with ");
              String _name_6 = broad.getName();
              final String msg_2 = (_plus_8 + _name_6);
              this.error(msg_2, o, f, "type");
            }
          }
        }
      }
    }
  }
  
  @Check(CheckType.FAST)
  public void checkReadonlyVar(final LocalVariable v) {
    boolean _isReadonly = v.isReadonly();
    boolean _not = (!_isReadonly);
    if (_not) {
      return;
    }
    Expression _init = v.getInit();
    boolean _tripleEquals = (_init == null);
    if (_tripleEquals) {
      String _name = v.getName();
      String _plus = ("Readonly local variable " + _name);
      final String msg = (_plus + " must be initialized on declaration");
      final Object parent = v.eContainer().eGet(v.eContainingFeature());
      if ((parent instanceof EList)) {
        this.error(msg, v.eContainer(), v.eContainingFeature(), ((EList<Action>) parent).indexOf(v), "readonly-not-init");
      } else {
        this.error(msg, v.eContainer(), v.eContainingFeature(), "readonly-not-init");
      }
    }
  }
  
  @Check(CheckType.FAST)
  public void checkVariableAssignment(final VariableAssignment va) {
    Expression _cardinality = va.getProperty().getTypeRef().getCardinality();
    boolean _tripleEquals = (_cardinality == null);
    if (_tripleEquals) {
      boolean _xifexpression = false;
      Variable _property = va.getProperty();
      if ((_property instanceof Property)) {
        Variable _property_1 = va.getProperty();
        _xifexpression = ((Property) _property_1).isReadonly();
      } else {
        boolean _xifexpression_1 = false;
        Variable _property_2 = va.getProperty();
        if ((_property_2 instanceof LocalVariable)) {
          Variable _property_3 = va.getProperty();
          _xifexpression_1 = ((LocalVariable) _property_3).isReadonly();
        } else {
          _xifexpression_1 = false;
        }
        _xifexpression = _xifexpression_1;
      }
      final boolean readOnly = _xifexpression;
      if (readOnly) {
        String _name = va.getProperty().getName();
        final String msg = ("Assigning read-only variable " + _name);
        final Object parent = va.eContainer().eGet(va.eContainingFeature());
        if ((parent instanceof EList)) {
          this.error(msg, va.eContainer(), va.eContainingFeature(), ((EList<Action>) parent).indexOf(va), "property-assign-readonly");
        } else {
          this.error(msg, va.eContainer(), va.eContainingFeature(), "property-assign-readonly");
        }
      }
    }
    this.checkType(va.getProperty(), va.getExpression(), va, ThingMLPackage.eINSTANCE.getVariableAssignment_Expression());
  }
  
  @Check(CheckType.FAST)
  public void checkLocalVariable(final LocalVariable lv) {
    Expression _init = lv.getInit();
    boolean _tripleNotEquals = (_init != null);
    if (_tripleNotEquals) {
      this.checkType(lv, lv.getInit(), lv, ThingMLPackage.eINSTANCE.getLocalVariable_Init());
    }
  }
  
  @Check(CheckType.FAST)
  public void checkProperty(final Property p) {
    Expression _init = p.getInit();
    boolean _tripleNotEquals = (_init != null);
    if (_tripleNotEquals) {
      this.checkType(p, p.getInit(), p, ThingMLPackage.eINSTANCE.getProperty_Init());
    }
  }
  
  @Check(CheckType.FAST)
  public void checkPropertyUsage(final Thing thing) {
    final List<Property> usedProperties = ThingHelper.allUsedProperties(thing);
    final Function1<Property, Boolean> _function = (Property p) -> {
      boolean _isDefined = AnnotatedElementHelper.isDefined(p, "ignore", "not-used");
      return Boolean.valueOf((!_isDefined));
    };
    final Procedure2<Property, Integer> _function_1 = (Property p, Integer i) -> {
      final boolean isUsed = usedProperties.contains(p);
      if ((!isUsed)) {
        String _name = p.getName();
        String _plus = ("Property " + _name);
        String _plus_1 = (_plus + " of Thing ");
        String _name_1 = thing.getName();
        String _plus_2 = (_plus_1 + _name_1);
        final String msg = (_plus_2 + " is never used. Consider removing (or using) it, or use @ignore \"not-used\".");
        this.warning(msg, p, ThingMLPackage.eINSTANCE.getNamedElement_Name(), "property-not-used");
      }
    };
    IterableExtensions.<Property>forEach(IterableExtensions.<Property>filter(thing.getProperties(), _function), _function_1);
    CompositeState _behaviour = thing.getBehaviour();
    boolean _tripleNotEquals = (_behaviour != null);
    if (_tripleNotEquals) {
      final Consumer<thingml.xthingml.thingML.State> _function_2 = (thingml.xthingml.thingML.State state) -> {
        final Function1<Property, Boolean> _function_3 = (Property p) -> {
          boolean _isDefined = AnnotatedElementHelper.isDefined(p, "ignore", "not-used");
          return Boolean.valueOf((!_isDefined));
        };
        final Procedure2<Property, Integer> _function_4 = (Property p, Integer i) -> {
          final boolean isUsed = usedProperties.contains(p);
          if ((!isUsed)) {
            String _name = p.getName();
            String _plus = ("Property " + _name);
            String _plus_1 = (_plus + " of Thing ");
            String _name_1 = thing.getName();
            String _plus_2 = (_plus_1 + _name_1);
            final String msg = (_plus_2 + " is never used. Consider removing (or using) it, or use @ignore \"not-used\".");
            this.warning(msg, p, ThingMLPackage.eINSTANCE.getNamedElement_Name(), "property-not-used");
          }
        };
        IterableExtensions.<Property>forEach(IterableExtensions.<Property>filter(state.getProperties(), _function_3), _function_4);
      };
      CompositeStateHelper.allContainedStatesIncludingSessions(thing.getBehaviour()).forEach(_function_2);
    }
  }
}
