package org.eclipse.gemoc.executionframework.property.monitor.esper.properties;

import java.util.List;
import java.util.Map;
import org.eclipse.gemoc.executionframework.property.model.property.Between;
import org.eclipse.gemoc.executionframework.property.model.property.Existence;
import org.eclipse.gemoc.executionframework.property.monitor.esper.TruthValue;
import org.eclipse.gemoc.executionframework.property.monitor.esper.properties.AbstractTemporalProperty;
import org.eclipse.viatra.query.patternlanguage.emf.specification.SpecificationBuilder;
import org.eclipse.viatra.query.runtime.api.IQuerySpecification;
import org.eclipse.xtend2.lib.StringConcatenation;

@SuppressWarnings("all")
public class ExistsPBetweenQAndR extends AbstractTemporalProperty {
  private final Existence exists;
  
  private final IQuerySpecification<?> p;
  
  private final IQuerySpecification<?> q;
  
  private final IQuerySpecification<?> r;
  
  public ExistsPBetweenQAndR(final SpecificationBuilder builder, final String name, final Existence exists, final Between between) {
    super(builder, name);
    this.exists = exists;
    this.p = builder.getOrCreateSpecification(exists.getPattern());
    this.q = builder.getOrCreateSpecification(between.getLowerBoundPattern());
    this.r = builder.getOrCreateSpecification(between.getUpperBoundPattern());
    this.queries.put(this.p.getFullyQualifiedName(), this.p);
    this.queries.put(this.q.getFullyQualifiedName(), this.q);
    this.queries.put(this.r.getFullyQualifiedName(), this.r);
  }
  
  @Override
  protected String getStatementString() {
    final String pFqn = this.getFqn(this.p);
    final String qFqn = this.getFqn(this.q);
    final String rFqn = this.getFqn(this.r);
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("select * from ");
    String _name = this.getName();
    _builder.append(_name);
    _builder.newLineIfNotEmpty();
    _builder.append("match_recognize (");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("measures P as P, Q as Q, EoE as EoE");
    _builder.newLine();
    _builder.append("\t");
    String _pattern = this.getPattern();
    _builder.append(_pattern, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("define");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("P as P.");
    _builder.append(pFqn, "\t\t");
    _builder.append("? is not null,");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("nPR as nPR.");
    _builder.append(pFqn, "\t\t");
    _builder.append("? is null and nPR.");
    _builder.append(rFqn, "\t\t");
    _builder.append("? is null,");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("Q as Q.");
    _builder.append(qFqn, "\t\t");
    _builder.append("? is not null,");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("R as R.");
    _builder.append(rFqn, "\t\t");
    _builder.append("? is not null,");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("EoE as EoE.executionAboutToStop? is not null");
    _builder.newLine();
    _builder.append(")");
    _builder.newLine();
    final String result = _builder.toString();
    return result;
  }
  
  private String getPattern() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("pattern (EoE | Q nPR* (P nPR*)* (R | EoE))");
    _builder.newLine();
    final String pattern = _builder.toString();
    return pattern;
  }
  
  @Override
  protected TruthValue getStatus(final Map<String, List<Map<?, ?>>> events) {
    final List<Map<?, ?>> lQ = events.get("Q");
    final boolean reachedQ = (!((lQ == null) || lQ.isEmpty()));
    if (reachedQ) {
      final List<Map<?, ?>> lEoE = events.get("EoE");
      final boolean reachedEoE = (!((lEoE == null) || lEoE.isEmpty()));
      if (reachedEoE) {
        return TruthValue.SATISFIED;
      } else {
        final List<Map<?, ?>> lP = events.get("P");
        int _xifexpression = (int) 0;
        if ((lP == null)) {
          _xifexpression = 0;
        } else {
          _xifexpression = lP.size();
        }
        final int nP = _xifexpression;
        TruthValue _xifexpression_1 = null;
        if (((nP >= this.exists.getMin()) && (nP <= this.exists.getMax()))) {
          _xifexpression_1 = TruthValue.UNKNOWN;
        } else {
          _xifexpression_1 = TruthValue.VIOLATED;
        }
        return _xifexpression_1;
      }
    } else {
      return TruthValue.SATISFIED;
    }
  }
}
