package org.eclipse.gemoc.executionframework.property.tests

import java.io.File
import java.io.FileOutputStream
import java.io.PrintWriter
import java.text.SimpleDateFormat
import java.util.ArrayList
import java.util.Calendar
import java.util.List
import java.util.Set
import org.eclipse.gemoc.execution.sequential.javaengine.tests.wrapper.JavaEngineWrapper
import org.eclipse.gemoc.executionframework.property.monitor.manager.PropertyManager
import org.eclipse.gemoc.executionframework.test.lib.IEngineWrapper
import org.eclipse.gemoc.executionframework.test.lib.IExecutableModel
import org.eclipse.gemoc.executionframework.test.lib.ILanguageWrapper
import org.eclipse.gemoc.executionframework.test.lib.impl.TestHelper
import org.eclipse.gemoc.executionframework.test.lib.impl.TestModel
import org.eclipse.gemoc.trace.commons.model.trace.Step
import org.eclipse.gemoc.xdsmlframework.api.core.IExecutionEngine
import org.eclipse.gemoc.xdsmlframework.api.engine_addon.IEngineAddon
import org.eclipse.xtend.lib.annotations.Data
import org.junit.AfterClass
import org.junit.BeforeClass

import static org.eclipse.gemoc.executionframework.property.tests.Util.*

abstract class AbstractMonitoringMemoryBenchmarkTestSuite extends AbstractMonitoringTestSuite {

	// Common to all tests
	static var File outputFolder
	static var File outputCSV
	static var PrintWriter outputCSVWriter
	static var FileOutputStream outputCSVStream

	abstract def String getSemanticsPlugin()

	@BeforeClass
	def static void before() {
		// Create output folder in test project
		val Calendar currentDate = Calendar::getInstance();
		val SimpleDateFormat formatter = new SimpleDateFormat("dd-MM-YYYY_HH-mm-ss")
		val String dateNow = formatter.format(currentDate.getTime())
		outputFolder = new File("output_time" + "_" + dateNow)
		if (!outputFolder.exists)
			outputFolder.mkdir

		// Prepare CSV file in output folder
		outputCSV = new File(outputFolder, "resultsMemory.csv")
		outputCSVStream = new FileOutputStream(outputCSV)
		outputCSVWriter = new PrintWriter(outputCSVStream, true)
	}

	@AfterClass
	def static void after() {
		outputCSVStream.close
		outputCSVWriter.close
	}

	new( /*int size,*/ String model, int scenarioID) {
		super( /*size,*/ model, scenarioID)
	}

	@Data
	static class BenchResult1 {
		List<List<Long>> allMemoryUsages 
	}

	@Data
	static class BenchResult2 {
		List<Long> memoryUsages
	}

	def IEngineAddon getMemoryMonitoringAddon(ArrayList<Long> memoryUsages) {
		return new IEngineAddon() {
			
			override stepExecuted(IExecutionEngine<?> engine, Step<?> stepExecuted) {
				val m = Util::currentlyUsedMemory
				memoryUsages.add(m)
			}

			override aboutToExecuteStep(IExecutionEngine<?> engine, Step<?> stepToExecute) {
				val m = Util::currentlyUsedMemory
				memoryUsages.add(m)
			}
		}
	}

	def BenchResult1 runBench(IEngineWrapper engine, ILanguageWrapper language, IExecutableModel model,
		String propertyFilepath) {
		val allMemoryUsages = new ArrayList<List<Long>>
		val Set<IEngineAddon> addons = if (propertyFilepath !== null && !propertyFilepath.empty) {
				val propertyManager = new PropertyManager
				propertyManager.addProperty(propertyFilepath, true)
				#{propertyManager}
			} else {
				#{}
			}
		for (x : 1 .. 30) {
			val memoryUsages = new ArrayList<Long>
			allMemoryUsages.add(memoryUsages)
			val Set<IEngineAddon> actualAddons = newHashSet => [it.addAll(addons)]
			actualAddons.add(getMemoryMonitoringAddon(memoryUsages))
			TestHelper::testWithJob(engine, language, #{}, actualAddons, model, true)
			cleanup(getSemanticsPlugin())
		}
		return new BenchResult1(allMemoryUsages.subList(allMemoryUsages.size - 11, allMemoryUsages.size))
	}

	def BenchResult2 testGeneric(String name, IEngineWrapper engine, ILanguageWrapper language, String plugin,
		String folder, String model, int propertyID) {

		val property = findProperty(model, plugin, folder, propertyID)
		val propertyFilepath = if(property.isPresent) property.get else ""
		val result1 = runBench(engine, language, new TestModel(plugin, folder, model, "", ""), propertyFilepath)
		val numbers = result1.allMemoryUsages

		println('''«name» - «model» - scenario «propertyID» :''')
		println("- median: " + getListMedian(numbers))

		return new BenchResult2(getListMedian(numbers))
	}

	abstract def ILanguageWrapper getDSL()

	def BenchResult2 testMonitoring(String plugin, String folder, String model, int propertyID) {
		val engine = new JavaEngineWrapper()
		return testGeneric("monitoring", engine, DSL, plugin, folder, model, propertyID)
	}

	override genericInternalTest(String plugin, String folder, String model, int propertyID) {
		val line = new CSVLineMemory

		line.modelName = model
		line.propertyID = propertyID

		val resultMonitoring = testMonitoring(plugin, folder, model, propertyID)

		line.memoryUsages = resultMonitoring.memoryUsages

		outputCSVWriter.println(line.customToString)
	}
}
