package org.eclipse.gemoc.executionframework.property.tests

import java.io.File
import java.io.FileOutputStream
import java.io.PrintWriter
import java.text.SimpleDateFormat
import java.util.ArrayList
import java.util.Calendar
import java.util.List
import java.util.Set
import org.eclipse.gemoc.execution.sequential.javaengine.tests.wrapper.JavaEngineWrapper
import org.eclipse.gemoc.executionframework.property.monitor.manager.PropertyManager
import org.eclipse.gemoc.executionframework.test.lib.IEngineWrapper
import org.eclipse.gemoc.executionframework.test.lib.IExecutableModel
import org.eclipse.gemoc.executionframework.test.lib.ILanguageWrapper
import org.eclipse.gemoc.executionframework.test.lib.impl.TestHelper
import org.eclipse.gemoc.executionframework.test.lib.impl.TestModel
import org.eclipse.gemoc.xdsmlframework.api.engine_addon.IEngineAddon
import org.eclipse.xtend.lib.annotations.Data
import org.junit.AfterClass
import org.junit.BeforeClass

import static org.eclipse.gemoc.executionframework.property.tests.Util.*

abstract class AbstractMonitoringTimeBenchmarkTestSuite extends AbstractMonitoringTestSuite {

	// Common to all tests
	static var File outputFolder
	static var File outputCSV
	static var PrintWriter outputCSVWriter
	static var FileOutputStream outputCSVStream

	abstract def String getSemanticsPlugin()

	@BeforeClass
	def static void before() {
		// Create output folder in test project
		val Calendar currentDate = Calendar::getInstance();
		val SimpleDateFormat formatter = new SimpleDateFormat("dd-MM-YYYY_HH-mm-ss")
		val String dateNow = formatter.format(currentDate.getTime())
		outputFolder = new File("output_time" + "_" + dateNow)
		if (!outputFolder.exists)
			outputFolder.mkdir

		// Prepare CSV file in output folder
		outputCSV = new File(outputFolder, "resultsTime.csv")
		outputCSVStream = new FileOutputStream(outputCSV)
		outputCSVWriter = new PrintWriter(outputCSVStream, true)
		outputCSVWriter.println(CSVLineTime::columnNames)
	}

	@AfterClass
	def static void after() {
		outputCSVStream.close
		outputCSVWriter.close
	}

	new(String model, int scenarioID) {
		super(model, scenarioID)
	}

	@Data
	static class BenchResult1 {
		int amountSteps
		List<Long> executionTimes
	}

	@Data
	static class BenchResult2 {
		int amountSteps
		Long executionTime
		Double variance
	}

	def BenchResult1 runBench(IEngineWrapper engine, ILanguageWrapper language, IExecutableModel model, String propertyFilepath) {
		val executionTimes = new ArrayList<Long>
		var boolean haveAmounts = false
		var int amountSteps
		val Set<IEngineAddon> addons = if (propertyFilepath !== null && !propertyFilepath.empty) {
			val propertyManager = new PropertyManager
			propertyManager.addProperty(propertyFilepath, true)
			#{propertyManager}
		} else {
			#{}
		}
		for (x : 1 .. 30) {
			val testResult = TestHelper::testWithJob(engine, language, #{}, addons, model, true)
			executionTimes.add(testResult.executionDuration)
			if (!haveAmounts) {
				amountSteps = testResult.amountOfStepsExecuted
				haveAmounts = true
			}
			cleanup(getSemanticsPlugin())
		}
		return new BenchResult1(amountSteps, executionTimes.subList(executionTimes.size - 11, executionTimes.size))
	}

	def BenchResult2 testGeneric(String name, IEngineWrapper engine, ILanguageWrapper language, String plugin,
		String folder, String model, String propertyFilepath) {

		val result1 = runBench(engine, language, new TestModel(plugin, folder, model, "", ""), propertyFilepath)
		val numbers = result1.executionTimes

		val mean = mean(numbers)
		val variance = meanDouble(numbers.map[n|(0.000001 * (n - mean)) * (0.000001 * (n - mean))])

		println('''«name» - «model» - scenario «propertyID» :''')
		println("- numbers: " + numbers)
		println("- mean: " + mean)
		println("- variance: " + variance)

		return new BenchResult2(result1.amountSteps, mean, variance)
	}

	abstract def ILanguageWrapper getDSL()

	def BenchResult2 testMonitoring(String plugin, String folder, String model, String propertyFilepath) {
		val engine = new JavaEngineWrapper()
		return testGeneric("monitoring", engine, DSL, plugin, folder, model, propertyFilepath)
	}

	override genericInternalTest(String plugin, String folder, String model, int propertyID) {
		if (propertyID > -2) {
			val line = new CSVLineTime
			line.modelName = model
			line.modelSize = Integer.parseInt(model.split("_").findFirst[s|s.startsWith("x") || s.endsWith("c")].replace("x", "").replace("c", ""))
			val propertyFilepath = findProperty(model, plugin, folder, propertyID).orElse("")
			line.property = propertyFilepath.split("/").last.split("\\.").head
			val resultMonitoring = testMonitoring(plugin, folder, model, propertyFilepath)
			line.executionTime = resultMonitoring.executionTime
			line.variance = resultMonitoring.variance
			line.amountSteps = resultMonitoring.getAmountSteps
			outputCSVWriter.println(line.customToString)
		} else {
			testMonitoring(plugin, folder, model, "")
		}
	}
}
