package org.eclipse.gemoc.executionframework.property.tests

import java.util.Collection
import java.util.List
import org.eclipse.gemoc.executionframework.property.tests.utils.PDETestResultsCollector
import org.junit.Rule
import org.junit.Test
import org.junit.rules.TemporaryFolder
import org.junit.runner.RunWith
import org.junit.runners.Parameterized
import org.junit.runners.Parameterized.Parameter
import org.junit.runners.Parameterized.Parameters
import java.io.File

@RunWith(Parameterized)
class MonitoringTimeBenchmarkTestSuiteLauncher {

	@Parameters
	static def Collection<Object[]> data() {
		return TestData::data
	}

	@Parameter
	public var String testClass

	@Parameter(1)
	public var String modelName

	@Parameter(2)
	public var int propertyId

	@Rule
	public TemporaryFolder tmpFolderCreator = new TemporaryFolder();

	/*
	 * Fill in the path to your Java home, gemoc launcher
	 * (e.g. "/home/<username>/Downloads/gemoc-studio/plugins/org.eclipse.equinox.launcher_1.5.0.v20180512-1130.jar"),
	 * path to the workspace containing the configuration for the test suite
	 * (i.e. where you executed the BenchmarkSingleJVMTestSuite manually),
	 * and the path where you want to write the output of the evaluation.
	 */
	static val int port = 7777
	static val String minMemory = "10g"
	static val String maxMemory = "10g"
	static val String javaHome = "/home/xp/jdk-10.0.2/bin/java"
	static val String gemocPath = "/home/xp/Documents/gemoc-studio_monitoring/plugins/org.eclipse.equinox.launcher_1.5.0.v20180512-1130.jar"
	static val String wsPath = "file:/home/xp/Documents/gemoc-studio_monitoring/junit-workspace"

	static def String prepareProperty(String key, String value) {
		return '''-D«key»=«value»'''
	}

//	@Before
//	def void prepareOutput() {
//		// Create output folder
//		val languageSimpleName = language.name.substring(language.name.lastIndexOf(".")+1)
//		val modelSimpleName = modelName.substring(0,modelName.lastIndexOf("."))
//		val outputFolder = new File(outputPath + "/" + languageSimpleName + "_" + modelSimpleName)
//		if (!outputFolder.exists)
//			outputFolder.mkdir
//		outputFolderPath = outputFolder.absolutePath
//	}
	def void log(String s) {
		println("### [" + modelName + "_" + propertyId + "] " + s)
	}

	@Test
	def void test() {
		// These params are completely specific to an environment,
		// and they need the PDE test suite (there, BenchmarkSingleJVMTestSuite) to have
		// been executed at least one before via Eclispe, so that its conf files
		// are ready.

		val List<String> params = #[ // "sudo", "ionice", "-c", "2", "-n", "0", "nice", "-19",
		javaHome, "-Xms" + minMemory, "-Xmx" + maxMemory, "-Declipse.pde.launch=true",
			"-Declipse.p2.data.area=@config.dir/p2", "--add-modules=ALL-SYSTEM", "-Dfile.encoding=UTF-8",
			prepareProperty("modelNameProperty", modelName), prepareProperty("propertyIdProperty", propertyId.toString),
			"-classpath", "/home/xp/Documents/gemoc_studio-monitoring/plugins/org.eclipse.equinox.launcher_1.5.0.v20180512-1130.jar",
			"org.eclipse.equinox.launcher.Main",
			"-os", "linux", "-ws", "gtk", "-arch", "x86_64", "-nl", "fr_FR", "-consoleLog", "-version", "3", "-port", port.toString,
			"-testLoaderClass",	"org.eclipse.jdt.internal.junit4.runner.JUnit4TestLoader",
			"-loaderpluginname", "org.eclipse.jdt.junit4.runtime",
			"-test", "org.eclipse.gemoc.executionframework.property.tests.ad.ActivityDiagramMonitoringTimeBenchmarkingTest:test",
//			"-test", "org.eclipse.gemoc.executionframework.property.tests.thingml.ThingMLMonitoringTimeBenchmarkingTest:test",
//			"-test", "org.eclipse.gemoc.executionframework.property.tests.minijava.MiniJavaMonitoringTimeBenchmarkingTest:test",
			"-application",	"org.eclipse.pde.junit.runtime.uitestapplication",
			"-product org.eclipse.platform.ide",
			"-testApplication",	"org.eclipse.ui.ide.workbench",
			"-data", "/home/xp/Documents/gemoc_studio-monitoring/junit-workspace",
			"-configuration", "file:/home/xp/Documents/gemoc_studio-monitoring/workspace/.metadata/.plugins/org.eclipse.pde.core/pde-junit/",
			"-dev", "file:/home/xp/Documents/gemoc_studio-monitoring/workspace/.metadata/.plugins/org.eclipse.pde.core/pde-junit/dev.properties",
			"-os", "linux", "-ws", "gtk",
			"-arch", "x86_64", "-nl", "fr_FR", "-consoleLog", "-testpluginname",
			"org.eclipse.gemoc.executionframework.property.tests"]

		val PDETestResultsCollector collector = new PDETestResultsCollector("listening for measure")

		// Start Junit listener in separate job
		log("Start dummy junit listener")
		val junitListener = new Runnable() {
			override run() {
				collector.run(port);
			}
		}
		val junitListenerThread = new Thread(junitListener)
		junitListenerThread.start

		// Run actual test
		log("Start test in dedicated JVM")
		val f = new File("/home/xp/Documents/gemoc_studio-monitoring/junit-workspace/.metadata/.plugins/org.eclipse.core.resources")
		f.listFiles.filter[s|s.name.endsWith(".snap")].forEach[s|s.delete]
		val ProcessBuilder processBuilder = new ProcessBuilder(params)
		val Process process = processBuilder.inheritIO.start()
		process.waitFor()
		
		// Finish test listener
		log("Kill dummy junit listener")
		junitListenerThread.stop
	}
}
